package com.abcinc.phonelist.test;

import com.abcinc.phonelist.ContactBean;
import com.meterware.httpunit.GetMethodWebRequest;
import com.meterware.httpunit.SubmitButton;
import com.meterware.httpunit.TableCell;
import com.meterware.httpunit.WebConversation;
import com.meterware.httpunit.WebForm;
import com.meterware.httpunit.WebLink;
import com.meterware.httpunit.WebRequest;
import com.meterware.httpunit.WebResponse;
import com.meterware.httpunit.WebTable;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * Test case fixture for the delete function of the phone list
 * application. This fixture contains several tests, each of which is
 * expressed in a method with a name of the form testXXX.
 *
 * @author Wellie Chao
 */
public class DeleteTest extends TestCase {

	// configurable constants.
	private static String protocol = "http";
	private static String hostname = "localhost";
	private static int port = 8080;
	private static String showListPath = "/phonelist/showList.do";
	private static String newPath = "/phonelist/new.do";
	private static String deletePath = "/phonelist/delete.do";
	private static String randomCharacterSet = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";

	// variables
	private static Random random = new Random();
	private WebConversation webConversation = new WebConversation();

	// main method so class can run from command line.
	public static void main(String args[]) {
		junit.textui.TestRunner.run(suite());
	}

	public static TestSuite suite() {
		return new TestSuite(DeleteTest.class);
	}

	public DeleteTest(String s) {
		super(s);
	}

	// test that the showList page contains a "delete contacts" button.
	public void testShowListHasDeleteButton() throws Exception {
		String url = protocol + "://" + hostname + ":" + port + showListPath;
		WebRequest webRequest = new GetMethodWebRequest(url);
		WebResponse webResponse = webConversation.getResponse(webRequest);
		WebForm webForm = webResponse.getFormWithID("contactListForm");
		// test that the form action points to the delete action.
		String formAction = webForm.getAction();
		// strip off session state information in the URL if it is there
		// because we want to compare the path of the URL.
		if (formAction.indexOf(";jsessionid=") >= 0) {
			formAction = formAction.substring(0, formAction.indexOf(";jsessionid="));
		}
		assertTrue("Contact form points to wrong action", formAction.endsWith(deletePath));
		SubmitButton deleteButton = webForm.getSubmitButton("submitButton", "Delete Selected");
		// make sure the delete button exists.
		assertNotNull("Could not find delete button", deleteButton);
	}

	/**
	 * Test that we can delete a contact. This test performs the
	 * following actions:
	 * 
	 * [1] Create two new contacts with random values for the fields.
	 * [2] Make note of the number of contacts in the database.
	 * [3] Make a list of all of the contacts from the database.
	 * [4] Select the contacts we created in step 1.
	 * [5] Click on the delete button.
	 * [6] Determine the number of contacts in the database.
	 * [7] Compare the number in step 6 to the number in step 2.
	 * [8] Verify that the difference in step 7 is 2.
	 * [9] Verify that none of the other contacts were deleted.
	 */
	public void testDeleteContact() throws Exception {
		String url = null;
		WebRequest webRequest = null;
		WebResponse webResponse = null;
		WebForm webForm = null;
		SubmitButton saveButton = null;
		ContactBean contactBean1, contactBean2;
		
		url = protocol + "://" + hostname + ":" + port + newPath;
		webRequest = new GetMethodWebRequest(url);

		// create new contact #1
		webResponse = webConversation.getResponse(webRequest);
		webForm = webResponse.getFormWithID("contactForm");
		contactBean1 = generateRandomContact();
		setWebFormFromBean(webForm, contactBean1);
		saveButton = webForm.getSubmitButton("submitButton", "	 Save		");
		webResponse = webForm.submit(saveButton);

		// create new contact #2
		webResponse = webConversation.getResponse(webRequest);
		webForm = webResponse.getFormWithID("contactForm");
		contactBean2 = generateRandomContact();
		setWebFormFromBean(webForm, contactBean2);
		saveButton = webForm.getSubmitButton("submitButton", "	 Save		");
		webResponse = webForm.submit(saveButton);

		// get the list of contacts
		WebTable webTable = null;
		url = protocol + "://" + hostname + ":" + port + showListPath;
		webRequest = new GetMethodWebRequest(url);
		webResponse = webConversation.getResponse(webRequest);
		// now parse the results after saving the contact.
		webTable = webResponse.getTableStartingWith("Phone List");
		int rowCount = webTable.getRowCount();
		// subtract the three rows for title, column headings, and buttons.
		int numContacts = rowCount - 3;
		TableCell lastBodyCell = webTable.getTableCell(rowCount - 2, 0);
		if (lastBodyCell.getColSpan() > 1) {
			// this means the message that there were no contacts was
			// displayed.
			numContacts = 0;
		}
		// store all of the contacts in the list in our own list so we can
		// compare our version with the server version later.
		List contactBeanList = new ArrayList();
		for(int i = 0; i < numContacts; i++) {
			// first two rows are table title and heading columns
			int rowIndex = i + 2;
			ContactBean contactBean = getBeanFromTableRow(webTable, rowIndex);
			// either the contact matches one of the two we created, or it
			// does not. If it matches, save the ID value so we can pass the
			// array of ID values to the delete action. If it does not
			// match, save the contact information so we can compare the
			// list after the delete action has run.
			if (contactBean.equals(contactBean1)) {
				contactBean1.setId(contactBean.getId());
			} else if (contactBean.equals(contactBean2)) {
				contactBean2.setId(contactBean.getId());
			} else {
				contactBeanList.add(contactBean);
			}
		}
		assertTrue("Newly created contact not found in list of contacts", ((contactBean1.getId() != null) && (contactBean2.getId() != null)));
		webForm = webResponse.getFormWithID("contactListForm");
		// choose the two contacts we just created.
		String[] idParameterArray = new String[2];
		idParameterArray[0] = contactBean1.getId().toString();
		idParameterArray[1] = contactBean2.getId().toString();
		webForm.setParameter("id", idParameterArray);
		SubmitButton deleteButton = webForm.getSubmitButton("submitButton", "Delete Selected");
		// submit the form with the two contacts selected.
		webResponse = webForm.submit(deleteButton);
		// now parse the results after issuing the delete request.
		webTable = webResponse.getTableStartingWith("Phone List");
		int newRowCount = webTable.getRowCount();
		// subtract the three rows for title, column headings, and buttons.
		int newNumContacts = newRowCount - 3;
		lastBodyCell = webTable.getTableCell(newRowCount - 2, 0);
		if (lastBodyCell.getColSpan() > 1) {
			// this means the message that there were no contacts was
			// displayed.
			newNumContacts = 0;
		}
		assertTrue("Delete action did not remove exactly two contacts", (newNumContacts == (numContacts - 2)));
		// verify that none of the other contacts were accidentally
		// deleted. This test currently assumes that the contacts will
		// remain in the exact same order after the delete action. If that
		// is not the case, this test needs to be rewritten.
		for(int i = 0; i < newNumContacts; i++) {
			ContactBean contactBeanFromList = (ContactBean)contactBeanList.get(i);
			// first two rows are table title and heading columns
			int rowIndex = i + 2;
			ContactBean contactBeanFromTable = getBeanFromTableRow(webTable, rowIndex);
			assertTrue("Delete action modified another contact", contactBeanFromList.equals(contactBeanFromTable));
		}
	}

	// generate a random string of the specified length.
	private String getRandomString(int length) {
		StringBuffer randomStringBuffer = new StringBuffer();
		for(int i = 0; i < length; i++) {
			int index = random.nextInt(randomCharacterSet.length());
			randomStringBuffer.append(randomCharacterSet.substring(index, index + 1));
		}
		return randomStringBuffer.toString();
	}

	// generate a new contact with random values for the fields.
	private ContactBean generateRandomContact() {
		ContactBean contactBean = new ContactBean();
		contactBean.setLastName(getRandomString(8));
		contactBean.setFirstName(getRandomString(8));
		contactBean.setPhone(getRandomString(8));
		contactBean.setFax(getRandomString(8));
		contactBean.setEmail(getRandomString(8));
		return contactBean;
	}

	// set form values in a WebForm from a ContactBean
	private void setWebFormFromBean(WebForm webForm, ContactBean contactBean) {
		webForm.setParameter("lastName", contactBean.getLastName());
		webForm.setParameter("firstName", contactBean.getFirstName());
		webForm.setParameter("phone", contactBean.getPhone());
		webForm.setParameter("fax", contactBean.getFax());
		webForm.setParameter("email", contactBean.getEmail());
	}

	// populate a new ContactBean from data in the showList table.
	private ContactBean getBeanFromTableRow(WebTable webTable, int rowIndex) {
		ContactBean contactBean = new ContactBean();
		TableCell tableCell = null;

		// second column is the "edit contact" link.
		tableCell = webTable.getTableCell(rowIndex, 1);
		WebLink webLink = tableCell.getLinkWith("Edit");
		String[] idParameterArray = webLink.getParameterValues("id");
		contactBean.setId(new Integer(idParameterArray[0]));

		// third column is last name.
		tableCell = webTable.getTableCell(rowIndex, 2);
		contactBean.setLastName(tableCell.asText());

		// fourth column is first name.
		tableCell = webTable.getTableCell(rowIndex, 3);
		contactBean.setFirstName(tableCell.asText());

		// fifth column is phone number.
		tableCell = webTable.getTableCell(rowIndex, 4);
		contactBean.setPhone(tableCell.asText());

		// sixth column is fax number.
		tableCell = webTable.getTableCell(rowIndex, 5);
		contactBean.setFax(tableCell.asText());

		// seventh column is email address.
		tableCell = webTable.getTableCell(rowIndex, 6);
		// email has an extra space at the beginning and an extra space at
		// the end, so we need to strip those.
		String columnValue = tableCell.asText();
		columnValue = columnValue.substring(1, columnValue.length() - 1);
		contactBean.setEmail(columnValue);

		return contactBean;
	}

}
