package com.abcinc.phonelist.test;

import com.meterware.httpunit.GetMethodWebRequest;
import com.meterware.httpunit.SubmitButton;
import com.meterware.httpunit.TableCell;
import com.meterware.httpunit.WebConversation;
import com.meterware.httpunit.WebForm;
import com.meterware.httpunit.WebLink;
import com.meterware.httpunit.WebRequest;
import com.meterware.httpunit.WebResponse;
import com.meterware.httpunit.WebTable;
import java.util.Random;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * Test case fixture for the new function of the phone list
 * application. This fixture contains several tests, each of which is
 * expressed in a method with a name of the form testXXX.
 *
 * @author Wellie Chao
 */
public class NewTest extends TestCase {

	// configurable constants.
	private static String protocol = "http";
	private static String hostname = "localhost";
	private static int port = 8080;
	private static String showListPath = "/phonelist/showList.do";
	private static String newPath = "/phonelist/new.do";
	private static String savePath = "/phonelist/save.do";
	private static String randomCharacterSet = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";

	// variables
	private static Random random = new Random();
	private WebConversation webConversation = new WebConversation();

	// main method so class can run from command line.
	public static void main(String args[]) {
		junit.textui.TestRunner.run(suite());
	}

	public static TestSuite suite() {
		return new TestSuite(NewTest.class);
	}

	public NewTest(String s) {
		super(s);
	}

	// test that the showList page contains a "new contact" link.
	public void testShowListHasNewContactLink() throws Exception {
		String url = protocol + "://" + hostname + ":" + port + showListPath;
		WebRequest webRequest = new GetMethodWebRequest(url);
		WebResponse webResponse = webConversation.getResponse(webRequest);
		WebTable webTable = webResponse.getTableStartingWith("Phone List");
		// test that the table with the phone list exists.
		assertNotNull("Phone List HTML table not found", webTable);
		// test that the last row of the table has a "new contact" link
		int rowCount = webTable.getRowCount();
		TableCell lastTableCell = webTable.getTableCell(rowCount - 1, 0);
		WebLink webLink = lastTableCell.getLinkWith("New Contact");
		String linkUrl = webLink.getURLString();
		// strip off session state information in the link if it is there
		// because we want to compare the path of the URL.
		if (linkUrl.indexOf(";jsessionid=") >= 0) {
			linkUrl = linkUrl.substring(0, linkUrl.indexOf(";jsessionid="));
		}
		assertTrue("Phone List has no new contact link", linkUrl.endsWith(newPath));
	}

	// test that the new contact page contains the required fields.
	public void testNewContactForm() throws Exception {
		String url = protocol + "://" + hostname + ":" + port + newPath;
		WebRequest webRequest = new GetMethodWebRequest(url);
		WebResponse webResponse = webConversation.getResponse(webRequest);
		WebForm webForm = webResponse.getFormWithID("contactForm");
		// test that the contact form exists.
		assertNotNull("Contact form not found", webForm);
		// test that the required fields exist on the form.
		assertTrue("Last Name field (lastName) missing from contact form", webForm.hasParameterNamed("lastName"));
		assertTrue("First Name field (firstName) missing from contact form", webForm.hasParameterNamed("firstName"));
		assertTrue("Phone field (phone) missing from contact form", webForm.hasParameterNamed("phone"));
		assertTrue("Fax field (fax) missing from contact form", webForm.hasParameterNamed("fax"));
		assertTrue("Email field (email) missing from contact form", webForm.hasParameterNamed("email"));
		// test that the form action points to the save action.
		String formAction = webForm.getAction();
		// strip off session state information in the URL if it is there
		// because we want to compare the path of the URL.
		if (formAction.indexOf(";jsessionid=") >= 0) {
			formAction = formAction.substring(0, formAction.indexOf(";jsessionid="));
		}
		assertTrue("Contact form points to wrong action", formAction.endsWith(savePath));
	}

	// test that we can save a new contact.
	public void testSaveNewContact() throws Exception {
		String url;
		WebRequest webRequest;
		WebResponse webResponse;
		
		// first get the existing list of contacts so we can count how
		// many contacts there are. We use the information later to
		// compare the number of contacts after saving a new contact.
		url = protocol + "://" + hostname + ":" + port + showListPath;
		webRequest = new GetMethodWebRequest(url);
		webResponse = webConversation.getResponse(webRequest);
		WebTable webTable = webResponse.getTableStartingWith("Phone List");
		int rowCount = webTable.getRowCount();
		// subtract the three rows for title, column headings, and buttons.
		int numContacts = rowCount - 3;
		TableCell lastBodyCell = webTable.getTableCell(rowCount - 2, 0);
		if (lastBodyCell.getColSpan() > 1) {
			// this means the message that there were no contacts was
			// displayed.
			numContacts = 0;
		}
		// now retrieve the new contact form page.
		url = protocol + "://" + hostname + ":" + port + newPath;
		webRequest = new GetMethodWebRequest(url);
		webResponse = webConversation.getResponse(webRequest);
		WebForm webForm = webResponse.getFormWithID("contactForm");
		// test that the contact form exists.
		assertNotNull("Contact form not found", webForm);
		String lastName = getRandomString(8);
		String firstName = getRandomString(8);
		String phone = getRandomString(8);
		String fax = getRandomString(8);
		String email = getRandomString(8);
		webForm.setParameter("lastName", lastName);
		webForm.setParameter("firstName", firstName);
		webForm.setParameter("phone", phone);
		webForm.setParameter("fax", fax);
		webForm.setParameter("email", email);
		// find the save button (rather than the cancel button).
		SubmitButton saveButton = webForm.getSubmitButton("submitButton", "		Save	 ");
		// make sure the save button exists.
		assertNotNull("Could not find save button", saveButton);
		webResponse = webForm.submit(saveButton);
		// now parse the results after saving the contact.
		webTable = webResponse.getTableStartingWith("Phone List");
		int newRowCount = webTable.getRowCount();
		// subtract the three rows for title, column headings, and buttons.
		int newNumContacts = newRowCount - 3;
		lastBodyCell = webTable.getTableCell(rowCount - 2, 0);
		if (lastBodyCell.getColSpan() > 1) {
			// this means the message that there were no contacts was
			// displayed.
			newNumContacts = 0;
		}
		assertTrue("Saving new contact did not increase contact count by 1", (newNumContacts == (numContacts + 1)));
		// test that the random data we generated matches the contact
		// information by trying to find a row with matching field values.
		Integer matchingContactId = null;
		for(int i = 0; i < newNumContacts; i++) {
			// first two rows are table title and heading columns
			int rowIndex = i + 2;
			// get the last name
			TableCell tableCell = webTable.getTableCell(rowIndex, 2);
			String columnValue = tableCell.asText();
			if (columnValue.equals(lastName)) {
				tableCell = webTable.getTableCell(rowIndex, 3);
				columnValue = tableCell.asText();
				if (!columnValue.equals(firstName)) continue;
				tableCell = webTable.getTableCell(rowIndex, 4);
				columnValue = tableCell.asText();
				if (!columnValue.equals(phone)) continue;
				tableCell = webTable.getTableCell(rowIndex, 5);
				columnValue = tableCell.asText();
				if (!columnValue.equals(fax)) continue;
				// email has an extra space at the beginning and an extra
				// space at the end, so we need to strip those.
				tableCell = webTable.getTableCell(rowIndex, 6);
				columnValue = tableCell.asText();
				columnValue = columnValue.substring(1, columnValue.length() - 1);
				if (!columnValue.equals(email)) continue;
				// get the ID value of the contact, stored in the "Edit" link
				// in the second column.
				tableCell = webTable.getTableCell(rowIndex, 1);
				WebLink webLink = tableCell.getLinkWith("Edit");
				String[] idParameterArray = webLink.getParameterValues("id");
				matchingContactId = new Integer(idParameterArray[0]);
				break;
			}
		}
		assertTrue("Newly created contact was not saved properly", (matchingContactId != null));
		// now delete the newly created contact.
		webForm = webResponse.getFormWithID("contactListForm");
		// choose the contact we just created.
		String[] idParameterArray = new String[1];
		idParameterArray[0] = matchingContactId.toString();
		webForm.setParameter("id", idParameterArray);
		SubmitButton deleteButton = webForm.getSubmitButton("submitButton", "Delete Selected");
		// submit the form with the two contacts selected.
		webResponse = webForm.submit(deleteButton);
	}

	// generate a random string of the specified length.
	private String getRandomString(int length) {
		StringBuffer randomStringBuffer = new StringBuffer();
		for(int i = 0; i < length; i++) {
			int index = random.nextInt(randomCharacterSet.length());
			randomStringBuffer.append(randomCharacterSet.substring(index, index + 1));
		}
		return randomStringBuffer.toString();
	}

}
